<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Validator;
use App\Models\User;
use App\Models\Otp;
use App\Models\Subscription;

use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;

class AuthController extends Controller
{
    //
    public function register(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'first_name'          =>  'required|max:40',
            'last_name'          =>  'required|max:40',
            'email'         =>  'email',
            'mobile_number' =>  'integer|min:8',
            'password' => [
                'required',
                'min:8',             // must be at least 8 characters in length
                'regex:/[a-z]/',      // must contain at least one lowercase letter
                'regex:/[A-Z]/',      // must contain at least one uppercase letter
                'regex:/[0-9]/',      // must contain at least one digit
                'regex:/[@$!%*#?&]/', // must contain a special character
            ],
            //'type'        =>  'required',
        ],[
            'first_name.required'   =>  trans('messages.F001'),
            'last_name.required'   =>  trans('messages.F001'),
            'first_name.max'   =>  trans('messages.F027'),
            'last_name.max'   =>  trans('messages.F027'),
            'email.email'        =>  trans('messages.F003'),
            'mobile_number.integers'          =>  trans('messages.F028'),
            'mobile_number.min'          =>  trans('messages.F030'),
            'password.required'          =>  trans('messages.F005'),
            'password.min'          =>  trans('messages.F029'),
            'password.regex'          =>  trans('messages.F031'),
            //'type.required'          =>  trans('messages.F034'),
        ]);

        $validator->after(function($validator) use($request) {

            $mobile_number = User::where('country_code',$request['country_code'])->where('mobile_number',$request['mobile_number'])->whereNotIn('status',['trashed'])->first();
            if ($mobile_number) {
                $validator->errors()->add('mobile_number', trans('messages.F033'));
            }
            if($request['email']){
                $email = User::where('email',$request['email'])->whereNotIn('status',['trashed'])->first();
                if ($email) {
                    $validator->errors()->add('email', trans('messages.F032'));
                }
            }
            
            
        });

        if ($validator->fails()) {
            // dd($validator->errors());
            $this->message = $validator->errors();
        }
        else
        {

            $addUser['password']        =   bcrypt($request->password);
            $addUser['first_name']            =   $request['first_name'];
            $addUser['last_name']            =   $request['last_name'];
            $addUser['email']           =   $request['email'];
            $addUser['is_otp_verified'] =   'no';
            $addUser['status']          =   'inactive';
            $addUser['type']            =   'user';
            $addUser['country_code']    =   $request['country_code'];
            $addUser['mobile_number']   =   $request['mobile_number'];
            // $addUser['type']            =   $request['type'];
            //dd($addUser);
            $data                       =   User::create($addUser);
            //dd($data);
            $otpUser['otp']             =   '1111';
            $otpUser['user_id']         =   $data['id'];
            $otp                        =   Otp::create($otpUser);

            $response = new \Lib\PopulateResponse($data);

            $this->data = $response->apiResponse();
            $this->status   = true;
            $this->message  = trans('messages.F007');
        }
            return $this->populateResponse();
    }

    public function login(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            //'country_code' => ['required'],
            //'mobile_number' => ['required'],
            'password' =>['required']
        ],[
            //'mobile_number.required'        =>  trans('messages.F022'),
            'password.required'             =>  trans('messages.F005'),
            //'country_code.required'         =>  trans('messages.F023'),
        ]);
        
        if(!empty($request->mobile_number)){
            $validator->after(function($validator) use(&$user, $request) {
                $credentials = request(['country_code','mobile_number', 'password']);

                if(!Auth::attempt($credentials))
                    $validator->errors()->add('mobile_number', trans('messages.F011'));

                
            });
        }
        if(!empty($request->email)){
            $validator->after(function($validator) use(&$user, $request) {
                $credentials = request(['email', 'password']);

                if(!Auth::attempt($credentials))
                    $validator->errors()->add('email', trans('messages.F011'));

                
            });
        }
        
        
        if ($validator->fails()) {
           $this->message = $validator->errors();
           //dd('hi');
        }else{
            if(!empty($request->mobile_number)){
                $userInactive = User::where('mobile_number', $request->mobile_number)->where('country_code', $request->country_code)->orderBy('created_at', 'desc')/*->where('is_completed','yes')*/->first();
            }
            if(!empty($request->email)){
                $userInactive = User::where('email', $request->email)->orderBy('created_at', 'desc')/*->where('is_completed','yes')*/->first();
            }
            
            if($userInactive['status'] == 'inactive'){

                $otpUser['otp']            =   '1
                111';
                $otpUser['user_id']         =   $userInactive['id'];
                $otp                        =   Otp::create($otpUser);

                $this->status   = true;
                $response = new \Lib\PopulateResponse($userInactive);

                $this->data = $response->apiResponse();
                return $this->populateResponse();

            }
            // dd('dd');
            if(!empty($request->mobile_number)){
                $user = User::where([
                    'country_code'=>$request->country_code,
                    'mobile_number'=>$request->mobile_number,
                    'status'=>'active'
                ])->first();
            }
            if(!empty($request->email)){
                $user = User::where([
                    'email'=>$request->email,
                    'status'=>'active'
                ])->first();
            }    

            $updateArr = array();
            // $updateArr['timezone'] = $request->timezone;
            if($request->device_token != "" && $request->device_type != "") {
                $updateArr['device_token'] = $request->device_token;
                $updateArr['device_type'] = $request->device_type == 'iphone' ? 'ios' : 'android';
            }
            if ($updateArr) {
                User::where('id',$user->id)->update($updateArr);
            }

            $userTokens = $user->tokens;

            if($userTokens){
                foreach($userTokens as $token) {
                    $token->revoke();   
                }
            }

            $tokenResult =  $user->createToken('MyApp');
            $token = $tokenResult->token;
            $token->save();
            $user['token'] = $tokenResult->accessToken;
            
            $response = new \Lib\PopulateResponse($user);

            $this->data = $response->apiResponse();
            $this->status   = true;
            $this->message  = trans('messages.F012');
            
        }
        return $this->populateResponse();
    }

    public function forgotPassword(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'country_code' => 'required',
            'mobile_number' => 'required',
        ],[
            'country_code.required' => trans('messages.F023'),
            'mobile_number.required' => trans('messages.F022'),
        ]);


        $validator->after(function($validator) use($request) {

            $user = User::where('country_code', $request->country_code)->where('mobile_number', $request->mobile_number)->first();

            if(empty($user)){
                $validator->errors()->add('mobile_number', trans('messages.F024'));
            }else{
                if($user->status == 'inactive'){
                    $validator->errors()->add('mobile_number', trans('messages.F025'));
                }
                if($user->status == 'trashed'){
                    $validator->errors()->add('mobile_number', trans('messages.F026'));
                }

            }

        });
        
        if ($validator->fails()) {
            // $this->type  = 'first';

            $this->message = $validator->errors();
        }
        else {
            $user = User::where('country_code', $request->country_code)->where('mobile_number', $request->mobile_number)->first();
            $otpUser['user_id'] = $user['id'];
            $otpUser['otp']     = '1111';

            $otp                        =   Otp::create($otpUser);

            $response = new \Lib\PopulateResponse($otpUser);
            $this->data = $response->apiResponse();
            $this->status   = true;
            $this->message  = 'Otp send successfully.';
        }
        
        return $this->populateResponse();     

    }

    public function otp(Request $request)
    {
        $validator = \Validator::make($request->all(), [
            'otp'         =>  'required',
            'user_id'     =>  'required'
        ],[
            'otp.required'   =>  trans('messages.F008'),
            'user.required'   =>  trans('messages.F008'),
        ]);

        $validator->after(function($validator) use($request) {
            $checkOTP = OTP::where([
                'user_id' => $request['user_id'],
                'otp' => $request['otp'],
            ])->latest()->first();
            // dd($checkOTP);
            if(empty($checkOTP)){
                $validator->errors()->add('error', trans('messages.F009'));
            }
            
        });

        if ($validator->fails()) {
            // $this->type  = 'first';

            $this->message = $validator->errors();
        }
        else
        {
            // dd('dd');
            $user = User::find($request['user_id']);
            // if($request->type == 'registration'){
            User::where('id', $request['user_id'])->update([
                'is_otp_verified' => 'yes',
                'status' => 'active',
            ]);
            $userTokens = $user->tokens;

            if($userTokens){
                foreach($userTokens as $token) {
                    $token->revoke();   
                }
            }

            $tokenResult =  $user->createToken('MyApp');
            $token = $tokenResult->token;
            $token->save();
            $user['token'] = $tokenResult->accessToken;
            //     $userTokens = $user->tokens;

            //     foreach($userTokens as $token) {
            //         $token->revoke();   
            //     }
            
            //     $tokenResult =  $user->createToken('MyApp');
            //     $token = $tokenResult->token;
            //     $token->save();
            //     $data = User::with('children','country')->where('id', $request['user_id'])->first();
            //     // dd($data);
            //     $data['token'] = $tokenResult->accessToken;
            //     $response = new \Lib\PopulateResponse($data);

            //     $this->data = $response->apiResponse();
            //     $this->status   = true;
            //     $this->message = trans('messages.F010');
            // }elseif ($request->type == 'forgotPassword') {
            $data = [];
            $users = User::find($request['user_id']);
            
            $response = new \Lib\PopulateResponse($user);

            $this->data = $response->apiResponse();
            $this->status   = true;
            $this->message = trans('messages.F010');
            // }
        }
            return $this->populateResponse();
        
    }

    public function resendOTP(Request $request)
    {
        $user = User::find($request->user_id);
        if($user){
            // OTP::where('user_id', $request->id)->latest()->first();

            // $data                       =   User::create($addUser);
            $otpUser['otp']             =   '1111';
            $otpUser['user_id']         =   $request->user_id;
            $otp                        =   OTP::create($otpUser);

            // $emailData['base_url']      = url('');
            // $emailData['site']          = 'ARABIC';
            // $emailData['email']         = $user['email'];
            // $emailData['name']          = ucfirst($user['name']);
            // // $emailData['userid']        = encrypt($user['id']);
            // $emailData['otp']           = $otpUser['otp'];
            // $statius                    = ___mail_sender($user['email'], $emailData['name'],'register',$emailData);
            // dd($statius);
            $data = [];
            $response = new \Lib\PopulateResponse($data);

            $this->data = $response->apiResponse();
            $this->status   = true;
            $this->message  = 'OTP resend successfully.';
        }
        return $this->populateResponse(); 
    }

    public function updatePassword(Request $request)
    {
        // dd($request->all());
        $validator = \Validator::make($request->all(), [
            'new_password'              =>  'required|min:6|max:15',
            'confirm_password'          =>  'required|same:new_password',
            
        ],[
            'new_password.required'     =>  trans('messages.F016'),
            'new_password.min'          =>  trans('messages.F017'),
            'new_password.max'          =>  trans('messages.F017'),
            'confirm_password.required'        =>  trans('messages.F018'),
            'confirm_password.same'        =>  trans('messages.F019'),
            
        ]);

        $validator->after(function($validator) use($request) {
            
            
        });

        if ($validator->fails()) {
            // $this->type  = 'first';

            $this->message = $validator->errors();
        }else{
            $input['password']  = bcrypt($request->new_password);
            // dd($request->user_id);
            User::where('id','=',$request->user_id)->update($input);
            $data = [];
            $response = new \Lib\PopulateResponse($data);
            $this->data = $response->apiResponse();
            $this->status   = true;
            $this->message  = trans('messages.F020');
        }
        // dd("ccc");

        return $this->populateResponse();  
    }

    public function planListing(Request $request)
    {

        $data['user'] = User::find(Auth::guard('api')->id());

        $plans = Subscription::where('status','active')->get();

        $response = new \Lib\PopulateResponse($plans);

        $this->data = $response->apiResponse();
        $this->status   = true;
        $this->message  = 'Plans fetched successfully.';
            return $this->populateResponse(); 
    }
}
